//+------------------------------------------------------------------+
//|                                                         ev_1.mq5 |
//|                                  Copyright 2024, MetaQuotes Ltd. |
//|                                             https://www.mql5.com |
//+------------------------------------------------------------------+
#property copyright "Copyright 2024, MetaQuotes Ltd."
#property link      "https://www.mql5.com"
#property version   "1.00"
#include <Expert\Signal\Other\CandlePatterns.mqh>
#define  __SIZE 5
input datetime __start = D'2022.01.01';
input datetime __stop = D'2023.01.01';
#property script_show_inputs
//+------------------------------------------------------------------+
//| Script program start function                                    |
//+------------------------------------------------------------------+
void OnStart()
{  //matrix filled with daily price changes across 5 applied prices
   matrix _m;
   _m.Init(__SIZE, __SIZE);
   _m.Fill(0.0);
   MqlRates _rates[];
   MqlDateTime _datetime;
   int _buffer_size = CopyRates(Symbol(), Period(), __start, __stop, _rates);
   PrintFormat(__FUNCSIG__+" buffered: %i",_buffer_size);
   if(_buffer_size >= 1)
   {  for(int i = 1; i < _buffer_size - 1; i++)
      {  TimeToStruct(_rates[i].time,_datetime);
         int _iii = int(_datetime.day_of_week)-1;
         if(_datetime.day_of_week == SUNDAY || _datetime.day_of_week == SATURDAY)
         {  _iii = 0;
         }
         for(int ii = 0; ii < __SIZE; ii++)
         {  if(ii == 0)
            {  _m[ii][_iii] += (_rates[i - 1].open - _rates[i].open);
            }
            else if(ii == 1)
            {  _m[ii][_iii] += (((_rates[i - 1].high+_rates[i - 1].low+(2.0*_rates[i - 1].close))/4.0) - ((_rates[i].high+_rates[i].low+(2.0*_rates[i].close))/4.0));
            }
            else if(ii == 2)
            {  _m[ii][_iii] += (((_rates[i - 1].high+_rates[i - 1].low)/2.0) - ((_rates[i].high+_rates[i].low)/2.0));
            }
            else if(ii == 3)
            {  _m[ii][_iii] += (_rates[i - 1].close - _rates[i].close);
            }
            else if(ii == 4)
            {  _m[ii][_iii] += (((_rates[i - 1].high+_rates[i - 1].low+_rates[i - 1].close)/3.0) - ((_rates[i].high+_rates[i].low+_rates[i].close)/3.0));
            }
         }
      }
   }
   PrintFormat(" for: %s, with: %s",Symbol(),EnumToString(Period()));
   //Print(" raw returns: \n", _m);
   //printf(" determinant is: %.2f", _m.Det());
   matrix _z = ZNorm(_m);
   matrix _cov_col = _z.Cov(false);
   matrix _e_vectors;
   vector _e_values;
   _cov_col.Eig(_e_vectors, _e_values);
   //Print(" eigen vectors: \n", _e_vectors);
   //Print(" eigen values: \n", _e_values);
   matrix _t = _e_vectors.Transpose();
   matrix _p = _m * _t;
   //Print(" projection: \n", _p);
   vector _max_row = _p.Max(0);
   vector _max_col = _p.Max(1);
   double _days[];
   _max_row.Swap(_days);
   PrintFormat(" best trade day is: %s", EnumToString(ENUM_DAY_OF_WEEK(ArrayMaximum(_days)+1)));
   ENUM_APPLIED_PRICE _price[__SIZE];
   _price[0] = PRICE_OPEN;
   _price[1] = PRICE_WEIGHTED;
   _price[2] = PRICE_MEDIAN;
   _price[3] = PRICE_CLOSE;
   _price[4] = PRICE_TYPICAL;
   double _prices[];
   _max_col.Swap(_prices);
   PrintFormat(" best applied price is: %s", EnumToString(_price[ArrayMaximum(_prices)]));
   PrintFormat(" worst trade day is: %s", EnumToString(ENUM_DAY_OF_WEEK(ArrayMinimum(_days)+1)));
   PrintFormat(" worst applied price is: %s", EnumToString(_price[ArrayMinimum(_prices)]));
   
}
//+------------------------------------------------------------------+
//| Z-Normalization                                                  |
//+------------------------------------------------------------------+
matrix ZNorm(matrix &M)
{  matrix _z;
   _z.Init(M.Rows(), M.Cols());
   _z.Copy(M);
   if(M.Rows() > 0 && M.Cols() > 0)
   {  double _std_min = (M.Max() - M.Min()) / (M.Rows() * M.Cols());
      if(_std_min > 0.0)
      {  double _mean = M.Mean();
         double _std = fmax(_std_min, M.Std());
         for(ulong i = 0; i < M.Rows(); i++)
         {  for(ulong ii = 0; ii < M.Cols(); ii++)
            {  _z[i][ii] = (M[i][ii] - _mean) / _std;
            }
         }
      }
   }
   return(_z);
}
//+------------------------------------------------------------------+
